@@ -20,8 +20,8 @@ import logging
 
 import telepathy
 import telepathy.constants
-import pymsn
-from pymsn.service.description.AB.constants import \
+import papyon
+from papyon.service.description.AB.constants import \
     ContactGeneral, ContactAnnotations
 
 from butterfly.handle import ButterflyHandleFactory
@@ -33,11 +33,11 @@ logger = logging.getLogger('Butterfly.Aliasing')
 
 class ButterflyAliasing(
         telepathy.server.ConnectionInterfaceAliasing,
-        pymsn.event.ContactEventInterface):
+        papyon.event.ContactEventInterface):
 
     def __init__(self):
         telepathy.server.ConnectionInterfaceAliasing.__init__(self)
-        pymsn.event.ContactEventInterface.__init__(self, self.msn_client)
+        papyon.event.ContactEventInterface.__init__(self, self.msn_client)
 
     def GetAliasFlags(self):
         return telepathy.constants.CONNECTION_ALIAS_FLAG_USER_SET
@@ -62,24 +62,30 @@ class ButterflyAliasing(
                     alias = u""
                 contact = handle.contact
                 if contact is None or \
-                        (not contact.is_member(pymsn.Membership.FORWARD)):
+                        (not contact.is_member(papyon.Membership.FORWARD)):
                     handle.pending_alias = alias
                     continue
-                infos = {ContactGeneral.ANNOTATIONS : \
-                            {ContactAnnotations.NICKNAME : alias.encode('utf-8')}
+
+                new_alias = alias.encode("utf-8")
+                old_alias = contact.infos.get(ContactGeneral.ANNOTATIONS, {}).\
+                    get(ContactAnnotations.NICKNAME, None)
+                if new_alias == old_alias:
+                    continue
+
+                infos = {ContactGeneral.ANNOTATIONS :
+                            {ContactAnnotations.NICKNAME : new_alias}
                         }
-                self.msn_client.address_book.\
-                    update_contact_infos(contact, infos)
+                self.msn_client.address_book.update_contact_infos(contact, infos)
             else:
                 self.msn_client.profile.display_name = alias.encode('utf-8')
                 logger.info("Self alias changed to '%s'" % alias)
                 self.AliasesChanged(((ButterflyHandleFactory(self, 'self'), alias), ))
 
-    # pymsn.event.ContactEventInterface
+    # papyon.event.ContactEventInterface
     def on_contact_display_name_changed(self, contact):
         self._contact_alias_changed(contact)
 
-    # pymsn.event.ContactEventInterface
+    # papyon.event.ContactEventInterface
     def on_contact_infos_changed(self, contact, updated_infos):
         alias = updated_infos.get(ContactGeneral.ANNOTATIONS, {}).\
             get(ContactAnnotations.NICKNAME, None)
@@ -87,11 +93,11 @@ class ButterflyAliasing(
         if alias is not None or alias != "":
             self._contact_alias_changed(contact)
 
-    # pymsn.event.ContactEventInterface
+    # papyon.event.ContactEventInterface
     def on_contact_memberships_changed(self, contact):
         handle = ButterflyHandleFactory(self, 'contact',
                 contact.account, contact.network_id)
-        if contact.is_member(pymsn.Membership.FORWARD):
+        if contact.is_member(papyon.Membership.FORWARD):
             alias = handle.pending_alias
             if alias is not None:
                 infos = {ContactGeneral.ANNOTATIONS : \
@@ -112,8 +118,10 @@ class ButterflyAliasing(
             alias = unicode(display_name, 'utf-8')
         else:
             contact = handle.contact
-            if contact is None:
-                alias = unicode(handle.account, 'utf-8')
+            if handle.pending_alias is not None:
+                alias = handle.pending_alias
+            elif contact is None:
+                alias = handle.account
             else:
                 alias = contact.infos.get(ContactGeneral.ANNOTATIONS, {}).\
                     get(ContactAnnotations.NICKNAME, None)
@@ -134,6 +142,6 @@ class ButterflyAliasing(
             alias = contact.display_name
 
         alias = unicode(alias, 'utf-8')
-        logger.info("Contact %r alias changed to '%s'" % (handle, alias))
-        self.AliasesChanged(((handle, alias), ))
+        logger.info("Contact %s alias changed to '%s'" % (unicode(handle), alias))
+        self.AliasesChanged([(handle, alias)])
 
